package com.apobates.forum.core.impl.event.listener;

import com.apobates.forum.core.dao.TopicTagDao;
import com.apobates.forum.core.entity.proxy.TopicReplica;
import com.apobates.forum.core.impl.event.TopicPublishEvent;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationListener;
import org.springframework.stereotype.Component;

/**
 * 保存话题的标签
 * 即使失败也可通过话题内容重塑
 * @author xiaofanku
 * @since 20200521
 */
@Component
public class TopicTagListener implements ApplicationListener<TopicPublishEvent> {
    @Autowired
    private TopicTagDao topicTagDao;
    private final static Logger logger = LoggerFactory.getLogger(TopicTagListener.class);
    
    @Override
    public void onApplicationEvent(TopicPublishEvent event) {
        logger.info("[Event][Topic][4]话题标签处理开始");
        TopicReplica topic = event.getTopic();
        if (null == topic.getTages() || topic.getTages().isEmpty()) {
            logger.info("[Event][Topic][4]话题标签是空");
            return;
        }
        //所有标签
        topicTagDao.batchSave(topic.getId(), topic.getTages());
        logger.info("[Event][Topic][4]话题标签处理结束");
    }
}