package com.apobates.forum.letterbox.impl.event;

import com.apobates.forum.letterbox.dao.InboxDao;
import com.apobates.forum.letterbox.entity.ForumLetterReceiver;
import com.apobates.forum.letterbox.entity.Inbox;
import com.apobates.forum.letterbox.entity.proxy.ForumLetterReplica;
import java.util.Set;
import java.util.stream.Collectors;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationListener;
import org.springframework.stereotype.Component;

/**
 * 将信件投递到信件人的收件箱中
 * @author xiaofanku
 * @since 20200527
 */
@Component
public class ForumLetterPostInboxListener implements ApplicationListener<ForumLetterPostEvent>{
    @Autowired
    private InboxDao inboxDao;
    private final static Logger logger = LoggerFactory.getLogger(ForumLetterPostInboxListener.class);
    
    @Override
    public void onApplicationEvent(ForumLetterPostEvent event) {
        ForumLetterReplica letter = event.getLetter();
        logger.info("[Event][Letter][1]信件投递通知开始");
        if (letter.getReceivers().isEmpty()) {
            return;
        }
        Set<Inbox> letterEntryRecords = letter.getReceivers().stream().map((ForumLetterReceiver fr)->new Inbox(fr.getMember(), letter.getAuthor(), letter.getNickname(), letter.getId())).collect(Collectors.toSet());
        inboxDao.batchSave(letterEntryRecords);
        logger.info("[Event][Letter][1]信件投递通知结束");
    }
}